#ifdef WIN32
	//do something
#elif __linux__
	#include <sys/stat.h>
	#include <sys/types.h>
	#include <sys/ioctl.h>
	#include <sys/mount.h>
	#include <linux/hdreg.h>
#else
	#error: unknown OS
#endif


#include <iostream>
#include <fstream>
#include <string.h>

#include "Utils.h"


#ifdef WIN32
bool get_drive_info(HANDLE _handle, drive_info &drive_info_)
{
	DWORD size_ = 0;
	BOOL is_accessible_ =
		::DeviceIoControl(_handle
		, IOCTL_STORAGE_CHECK_VERIFY
		, 0
		, 0
		, 0
		, 0
		, &size_
		, 0);

	if (!is_accessible_)
	{
		return false;
	}

	DISK_GEOMETRY disk_geometry_;
	::memset(&disk_geometry_, 0, sizeof disk_geometry_);

	BOOL is_geometry_ =
		::DeviceIoControl(_handle
		, IOCTL_DISK_GET_DRIVE_GEOMETRY
		, 0
		, 0
		, &disk_geometry_
		, sizeof disk_geometry_
		, &size_
		, 0);

	if (!is_geometry_)
	{
		return false;
	}

	drive_info_._cylinder = disk_geometry_.Cylinders.QuadPart;
	drive_info_._type = fixed;
	drive_info_._tracks_per_cylinder = disk_geometry_.TracksPerCylinder;
	drive_info_._sectors_per_track = disk_geometry_.SectorsPerTrack;
	drive_info_._bytes_per_sector = disk_geometry_.BytesPerSector;

	drive_info_._lba_size = drive_info_._cylinder
		* drive_info_._bytes_per_sector
		* drive_info_._sectors_per_track
		* drive_info_._tracks_per_cylinder;

	return true;
}

#elif __linux__
bool get_drive_info(HANDLE _handle, drive_info &drive_info_)
{
	struct hd_geometry geometry;
	::memset(&geometry, 0, sizeof geometry);

	int result = ioctl(_handle, HDIO_GETGEO, &geometry);

	if (result < 0)
	{
		std::cout<<"ioctl error";
		return drive_info drive_info_();
	}

	drive_info drive_info_;

	drive_info_._cylinder = geometry.cylinders;
	drive_info_._type = fixed;
	drive_info_._tracks_per_cylinder = geometry.heads;
	drive_info_._sectors_per_track = geometry.sectors;


	long sector_size = 0;
	result = ioctl(_handle, BLKSSZGET, &sector_size); // BLKSSZGET /* Get block device sector size.  */

	if (result < 0)
	{
		std::cout<<"ioctl error";
		return drive_info drive_info_();
	}

	drive_info_._bytes_per_sector = sector_size;

	drive_info_._lba_size = drive_info_._cylinder
		* drive_info_._bytes_per_sector
		* drive_info_._sectors_per_track
		* drive_info_._tracks_per_cylinder;

	return true;
}

#else
	#error: unknown OS
#endif

void store_drive_geometry(drive_info &di)
{
	std::ofstream ofile("mbr_drive_geometry", std::ofstream::binary);

	ofile<<"_cylinder = "<<di._cylinder<<std::endl
		<<"_type = "<<di._type<<std::endl
		<<"_tracks_per_cylinder = "<<di._tracks_per_cylinder<<std::endl
		<<"_sectors_per_track = "<<di._sectors_per_track<<std::endl
		<<"_bytes_per_sector = "<<di._bytes_per_sector<<std::endl
		<<"_lba_size = "<<di._lba_size<<std::endl;

	ofile.close();
}